# from PySide6.QtCore import Qt
# from .text_note_widget import TextNoteWidget
#
#
# class MarkdownNoteWidget(TextNoteWidget):
#     def __init__(self, note, parent=None):
#         super(MarkdownNoteWidget, self).__init__(note, parent=parent)
#
#         self.setTextFormat(Qt.MarkdownText)
