animals = '''Ant
Antelope
Baboon
Bat
Beagle
Bear
Bird
Butterfly
Cat
Caterpillar
Chicken
Cow
Dog
Dolphin
Donkey
Eagle
Fish
Fly
Fox
Frog
Gerbil
Goose
Gopher
Gorilla
Heron
Honey Bee
Horn Shark
Horse
Ibis
Iguana
Impala
Jackal
Jaguar
Javanese
Jellyfish
Kakapo
Kangaroo
King Penguin
Kiwi
Koala
Lemming
Lemur
Leopard
Saola
Scorpion
Snake
Swan
Tuatara
Turkey
Zebra'''.split('\n')

fruits = '''Apple
Banana
Orange
Mango
Grapes
Pineapple
Pomegranate
Avocado
Coconut
Papaya
Watermelon
Dragonfruit
Strawberry
Blueberry
Blackberry
Gooseberry
Cherry
Apricot
Jackfruit
Kiwifruit
Lime
Peach
Pear
Tamarind
Custard-Apple
Fig
Guava
MuskMelon
Olives
Java Plum'''.split('\n')

adjectives = '''able
best
better
big
black
certain
clear
different
early
easy
economic
federal
free
full
good
great
grumpy
gifted
hard
high
human
hilarious
important
international
large
late
little
local
long
low
major
new
only
other
possible
public
real
recent
right
small
social
special
strong
sure
true
white
whole'''.split('\n')
